// the slime2:ready event is fired once
// indicates that the slime2 global variable is ready to use
addEventListener('slime2:ready', () => {
  slime2.widget.loadSettings('data-flex-chat.js', [
    defineSetting(
      [ 
        'Flexible Bubble Chat', 
        'melon (X @melonrbts)',
      ].join('\n'),
      'title',
      'text-display',
    ),
    stackSetting,
    generalSettings,
    filterSettings,
    fontSettings,
    fullSettings,
    compactSettings,
  ])
})

/************************
 * Setting Definitions *
 ************************/

function defineSetting(label, id, type, options) {
  return { label, id, type, ...options }
}

function defineOptions(options) {
  return options.map(option => {
    const [label, value] = option
    return { label, value }
  })
}

const stackSetting = defineSetting('Stack', 'stacks', 'select-input', {
  defaultValue: 'full',
  options: defineOptions(['full', 'compact'].map(label => [label, label])),
  description: 'Full stacks user on message, compact aligns user and message.',
})

const positionSettings = defineSetting('Position', 'position', 'group', {
  settings: [
    defineSetting('Direction', 'direction', 'select-input', {
      defaultValue: 'column',
      options: defineOptions(['column', 'row'].map(label => [label, label])),
      description: 'Direction that multiple messages line up.',
    }),
    defineSetting('Corner', 'corner', 'select-input', {
      defaultValue: 'bottom-left',
      options: defineOptions(
        ['top left', 'top right', 'bottom right', 'bottom left'].map(label => [
          label, label.replaceAll(' ', '-'),
        ]),
      ),
    }),
  ],
})

const timingSettings = defineSetting('Timing', 'timing', 'group', {
  settings: [
    defineSetting('Delay (seconds)', 'delay', 'number-input', {
      defaultValue: 0,
      min: 0,
      step: 1,
      description: 'Amount of time to delay message.',
    }),
    defineSetting('Expiration (seconds)', 'expiration', 'number-input', {
      defaultValue: 0,
      min: 0,
      step: 1,
      description: 'Amount of time to display message.',
    }),
    defineSetting('Maximum number of messages', 'max', 'number-input', {
      defaultValue: 10,
      min: 0,
      max: 200,
      step: 1,
      description: 'Maximum number of messages to display. FIFO.',
    }),
  ],
})

const animationSettings = defineSetting('Animations', 'animations', 'group', {
  settings: [
    defineSetting('Entrance animation', 'enter', 'select-input', {
      defaultValue: 'none',
      options: defineOptions(
        ['swipe up', 'swipe down', 'swipe left', 'swipe right', 'fade', 'none'].map(label => [
          label, label.replaceAll(' ', '-'),
        ])
      ),
    }),
    defineSetting('Exit animation', 'exit', 'select-input', {
      defaultValue: 'none',
      options: defineOptions(
        ['swipe up', 'swipe down', 'swipe left', 'swipe right', 'fade', 'none'].map(label => [
          label, label.replaceAll(' ', '-'),
        ])
      ),
    }),
  ],
})

const badgeSettings = defineSetting('Badges', 'badges', 'group', {
  settings: [
    defineSetting('Show badges', 'display', 'boolean-input', {
      defaultValue: true,
    }),
    defineSetting('Badge size (px)', 'size', 'number-input', {
      defaultValue: 18,
      min: 0,
      step: 1,
    }),
    defineSetting('Host badge', 'hostBadge', 'image-input', {
      description: 'Set host/broadcaster badge.',
    }),
    defineSetting('Moderator badge', 'modBadge', 'image-input', {
      description: 'Set moderator badge.',
    }),
    defineSetting('VIP badge', 'vipBadge', 'image-input', {
      description: 'Set VIP badge.',
    }),
    defineSetting('Founder badge', 'founderBadge', 'image-input', {
      description: 'Set founder badge.',
    }),
    defineSetting('Artist badge', 'artistBadge', 'image-input', {
      description: 'Set artist badge.',
    }),
  ],
})

const emoteSettings = defineSetting('Emotes', 'emotes', 'group', {
  settings: [
    defineSetting('Static emotes', 'static', 'boolean-input', {
      defaultValue: false,
      description:
        'When enabled, emotes are static.',
    }),
    defineSetting('Use dynamic emote sizing', 'dynamic', 'boolean-input', {
      defaultValue: true,
      description:
        'When enabled, emotes are larger when the chat message only contains emotes.',
    }),
    defineSetting('Dynamic size for multiple emotes (px)', 'medium', 'number-input', {
      defaultValue: 56,
      slider: true,
      min: 28,
      max: 112,
      step: 1,
      description: 'Messages contains only multiple emotes.',
    }),
    defineSetting('Dynamic size for single emote (px)', 'large', 'number-input', {
      defaultValue: 112,
      slider: true,
      min: 28,
      max: 112,
      step: 1,
      description: 'Message contains only one emote.',
    }),
    defineSetting('Dynamic emote sizing example', 'dynamicPreview', 'image-display', {
      url: 'https://i.imgur.com/vfHCWjB.png',
      alt: [
        'Three chat messages.',
        "1. Text and normal sized emote.",
        '2. Multiple medium-sized emotes.',
        "3. One large emote.",
      ].join('\n'),
   }),
  ],
})

const pronounSettings = defineSetting('Pronouns', 'pronouns', 'group', {
  settings: [
    defineSetting( 'Set pronouns here: https://pronouns.alejo.io/', 'link', 'text-display',
    ),
    defineSetting('Pronouns Display', 'display', 'select-input', {
      defaultValue: 'lowercase',
      options: defineOptions(['lowercase', 'capitalize', 'uppercase', 'hidden'].map(label => [label,label])),
      description: [
        'Examples:',
        '- Lowercase: she/her',
        '- Capitalize: She/Her',
        '- Uppercase: SHE/HER',
      ].join('\n'),
    }),
  ],
})

const generalSettings = defineSetting('General', 'general', 'group', {
  settings: [
    positionSettings,
    timingSettings,
    animationSettings,
    badgeSettings,
    emoteSettings,
    pronounSettings,
  ],
})

const botFilterSettings = defineSetting('Bot Filters', 'botFilters', 'group', {
  settings: [
    defineSetting('Bots to hide', 'names', 'text-input', {
      multiple: true,
      defaultValue: ['Sery_Bot', 'Nightbot', 'StreamElements', 'Streamlabs'],
      placeholder: 'Bot name',
      description: 'Bots to hide message from.',
    }),
    defineSetting('Command prefixes to hide', 'prefixes', 'text-input', {
      multiple: true,
      defaultValue: ['!'],
      placeholder: 'Command prefix',
      description:
        'Hide messages with specified prefixes.',
    }),
  ],
})

const userFilterSettings = defineSetting('User Filters', 'userFilters', 'group', {
  settings: [
    defineSetting('Show messages from these types of users', 'types', 'select-input', {
      multiple: true,
      defaultValue: ['all', 'subs', 'mods', 'vips', 'followers', 'artists'],
      options: defineOptions([
        ['everyone', 'all'],
        ['followers', 'followers'],
        ['subscribers', 'subs'],
        ['moderators', 'mods'],
        ['VIPs', 'vips'],
        ['artists', 'artists'],
      ]),
    }),
    defineSetting('Follow age (hours)', 'followHours', 'number-input', {
      defaultValue: 0,
      min: 0,
      step: 'any',
      description:
        "Amount of follow time to be considered a follower.",
    }),
    defineSetting('Always show messages from these users', 'users', 'text-input', {
      multiple: true,
      placeholder: 'Username',
    }),
  ],
})

const messageFilterSettings = defineSetting('Message Filters', 'messageFilters', 'group', {
  settings: [
    defineSetting('Hide these message types', 'types', 'select-input', {
      multiple: true,
      defaultValue: [],
      options: defineOptions([
        ['First-time chat', 'first'],
        ['/me message', 'action'],
        ['Messages with text', 'text'],
        ['Messages with emotes', 'emote'],
        ['Message with bits', 'cheer'],
        ['Replies', 'reply'],
        ['Highlighted', 'highlight'],
        ['Channel point redemption message', 'redeem'],
        ['Resub message', 'resub'],
        ['Announcements', 'announcement'],
      ]),
    }),
    defineSetting('Always hide messages containing these words', 'words', 'text-input', {
      multiple: true,
    }),
  ],
})

const filterSettings = defineSetting('Filters', 'filters', 'group', {
  settings: [botFilterSettings, userFilterSettings, messageFilterSettings],
})

const fontSettings = defineSetting('Font style', 'fontStyle', 'group', {
  settings:[
    defineSetting('Font', 'font', 'font-input'),
    defineSetting('Size (px)', 'size', 'number-input', {
      defaultValue: 20,
      min: 0,
      step: 1,
    }),
    defineSetting('Font weight', 'weight', 'select-input', {
      defaultValue: 800,
      options: defineOptions(['normal', 'bold'].map(label => [label,label])
      .concat(
        [100, 200, 300, 400, 500, 600, 700, 800, 900].map(value => [value.toString(), value,]),
      )),
    }),
    defineSetting('Outline style', 'outline', 'select-input', {
      defaultValue: 'none',
      options: defineOptions(
        [
          'outline 1',
          'outline 2',
          'none',
        ].map(label => [label, label.replaceAll(' ', '-')]),
      ),
      description: 'Outline 2 is thicker than outline 1.',
    }),
    defineSetting('Outline', 'outlineSection', 'select-input', {
      defaultValue: 'none',
      options: defineOptions([
        'username', 'message', 'both', 'none'].map(label => 
        [label, label])),
      description: 'Which message elements to outline.',
    }),
    defineSetting('Outline color', 'outlineColor', 'color-input', {
      defaultValue: '#3C3837',
    }),
  ],
})

const fullUserSettings = defineSetting('Full Username Style', 'fullUser', 'group', {
  settings: [
    defineSetting('Username background color', 'userBackgroundColor', 'select-input', {
      defaultValue: 'light',
      options: defineOptions([
        ['twitch light', 'light'],
        ['twitch dark', 'dark'],
        ['twitch', 'reg'],
        ['custom', 'custom'],
      ]),
      description:
        'Twitch Light/Dark are lightened/darkened versions of the username colors people have chosen in chat.',
    }),
    defineSetting('Custom username background color', 'customUserColor', 'color-input', {
      defaultValue: 'white',
      description: 'This will only be applied when you select "custom" above.',
    }),
    defineSetting('Custom username text color', 'customUserTextColor', 'color-input', {
      defaultValue: '#3C3837',
    }),
    defineSetting('Username rotation angle (deg)', 'userAngle', 'number-input', {
      slider: true,
      defaultValue: 0,
      min: -5,
      max: 5,
      step: 'any',
    }),
    defineSetting('Username border radius (px)', 'userBorderRadius', 'number-input', {
      slider: true,
      defaultValue: 25,
      min: 0,
      step: 1,
    }),
    defineSetting('Userame border width (px)', 'userBorderWidth', 'number-input', {
      slider: true,
      defaultValue: 2,
      min: 0,
      step: 1,
    }),
    defineSetting('Username border color', 'userBorderColor', 'color-input', {
      defaultValue: '#3C3837',
    }),
    defineSetting('Username vertical position (em)', 'userVertical', 'number-input', {
      slider: true,
      defaultValue: -0.5,
      min: -1,
      max: 1,
      step: 'any',
      description: 'Vertical offset of username. Expressed relative to font size.',
    }),
    defineSetting('Username horizontal position (em)', 'userHorizontal', 'number-input', {
      slider: true,
      defaultValue: 0.3,
      min: 0,
      max: 2,
      step: 'any',
      description: 'Horizontal offset of username. Expressed relative to font size.',
    }),
  ],
})

const fullMessageSettings = defineSetting('Full Message Style', 'fullMessage', 'group', {
  settings: [
    defineSetting('Custom message background color', 'customMessageColor', 'color-input', {
      defaultValue:'#FBF6F0',
    }),
    defineSetting('Custom message text color', 'customMessageTextColor', 'color-input', {
      defaultValue: '#3C3837',
    }),
    defineSetting('Message border radius (px)', 'messageBorderRadius', 'number-input', {
      slider: true,
      defaultValue: 25,
      min: 0,
      step: 1,
    }),
    defineSetting('Message border width (px)', 'messageBorderWidth', 'number-input', {
      slider: true,
      defaultValue: 2,
      min: 0,
      step: 1,
    }),
    defineSetting('Message border color', 'messageBorderColor', 'color-input', {
      defaultValue: '#3C3837',
    }), 
    defineSetting('Message horizontal position (em)', 'messageHorizontal', 'number-input', {
      slider: true,
      defaultValue: 0,
      min: 0,
      max: 2,
      step: 'any',
      description: 'Horizontal offset of message. Expressed relative to font size.',
    }),   
  ],
})

const fullSettings = defineSetting('Full chat', 'fullStyle', 'group', {
  settings: [fullUserSettings, fullMessageSettings],
})

const compactSettings = defineSetting('Compact chat', 'compactStyle', 'group', {
  settings: [
    defineSetting('Username color', 'userColor', 'select-input', {
      defaultValue: 'light',
      options: defineOptions([
        ['twitch light', 'light'],
        ['twitch dark', 'dark'],
        ['twitch', 'reg'],
        ['custom', 'custom'],
      ]),
      description:
        'Twitch Light/Dark are lightened/darkened versions of the username colors people have chosen in chat.',
    }),
    defineSetting('Custom username text color', 'customUserColor', 'color-input', {
      defaultValue: '#3C3837',
      description: 'This will only be applied when you select "custom" above.',
    }),
    defineSetting('Custom message text color', 'customMessageColor', 'color-input', {
      defaultValue: '#3C3837',
    }),
    defineSetting('Custom background color', 'customBackgroundColor', 'color-input', {
      defaultValue: '#00000000',
    }),
    defineSetting('Border radius (px)', 'compactBorderRadius', 'number-input', {
      slider: true,
      defaultValue: 0,
      min: 0,
      step: 1,
    }),
    defineSetting('Border width (px)', 'compactBorderWidth', 'number-input', {
      slider: true,
      defaultValue: 0,
      min: 0,
      step: 1,
    }),
    defineSetting('Border color', 'compactBorderColor', 'color-input', {
      defaultValue: '#00000000',
    }),     
  ],
})